import requests
import time
from typing import List, Dict, Any, Text
from tabelog_scraper.adapter.extractor.scraped_page import ScrapedDetailPage, ScrapedListPage


class ScrapingExtractor(object):
    def __init__(self, target_url, limit_page_count) -> None:
        self.target_url = target_url
        self.limit_page_count = limit_page_count
        self.page_count = 1

    def get_store_urls(self) -> List[str]:
        store_urls: List[str] = []
        while self._can_search():
            self.page_count = self.page_count + 1

            _wait_a_second()
            response: requests.Response = requests.get(self.target_url)
            scraped_list_page = ScrapedListPage(response.text)
            self.target_url = scraped_list_page.get_next_target_url()
            store_urls.extend(scraped_list_page.get_detail_urls())

        return store_urls

    def get_store(self, store_url) -> Dict[Text, Any]:
        response: requests.Response = requests.get(store_url)
        scraped_detail_page = ScrapedDetailPage(response.text, response.url)
        return scraped_detail_page.get_store()

    def _can_search(self) -> bool:
        if len(self.target_url) == 0:
            return False
        if self.page_count > self.limit_page_count:
            return False
        return True


def _wait_a_second() -> None:
    time.sleep(1)



from bs4 import BeautifulSoup
from typing import Dict, Text, Any, List


class ScrapedPage:
    def __init__(self, html: str):
        self.html: str = html

    def create_soup(self):
        # UTF-8にエンコードしないといけない
        return BeautifulSoup(self.html, 'html.parser')


class ScrapedListPage(ScrapedPage):
    def __init__(self, html: str):
        super().__init__(html)

    def get_detail_urls(self) -> List[str]:
        soup = self.create_soup()

        urls = []
        for a in soup.select('div.list-rst'):
            url = a.get('data-detail-url')
            urls.append(url)

        return urls

    def get_next_target_url(self):
        try:
            soup = self.create_soup()
            return soup.find(
                'a', class_='c-pagination__arrow--next').attrs['href']
        except AttributeError:
            return ''


class ScrapedDetailPage(ScrapedPage):
    def __init__(self, html: str, url: str):
        super().__init__(html)
        self.url: str = url

    def get_store(self) -> Dict[Text, Any]:
        soup = self.create_soup()

        result: Dict[Text, Any] = {}
        # 店名
        result["name"] = soup.select('.display-name > span')[0].text.strip()

        # 地理情報
        result["navigation"] = self.get_navigation()

        # レーティング
        result["rate"] = _convert_to_rate(
            soup.select('.rdheader-rating__score-val-dtl')[0].text)

        # 住所
        result["address"] = soup.select(
            '.rstinfo-table__address')[0].text.strip()

        # マップ画像
        result["address_image_url"] = soup.select(
            '.js-map-lazyload')[0].attrs['data-original']

        # URL
        result["url"] = self.url

        return result

    def get_navigation(self) -> str:
        soup = self.create_soup()
        breadcrumbs = soup.select('#location-breadcrumbs-wrap span')
        # 「ホーム > 食べログ」と、最後の店名は要らない
        # もしこの３つしかパンくずリストになかったら、空の文字列を返す
        if len(breadcrumbs) <= 3:
            return ''
        # 上記の不要な箇所を除外してから、テキストを抽出してカンマでつなぐ
        breadcrumbs = breadcrumbs[2: len(breadcrumbs)]
        breadcrumbs = breadcrumbs[0:-1]
        return ",".join(list(map(lambda b: b.text, breadcrumbs)))


def _convert_to_rate(rate_value: str) -> float:
    try:
        return float(rate_value)
    except ValueError:
        return 0.0


if __name__ == '__main__':
    import requests
    response = requests.get(
        'https://tabelog.com/tokyo/A1315/A131501/13194580/')
    detail_page = ScrapedDetailPage(response.text, response.url)
    print(detail_page.get_navigation())




from abc import ABCMeta, abstractmethod
from typing import Dict, Tuple, Text, Any, List
import inject


class Extractor(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_store_urls(self) -> List[str]:
        pass

    @abstractmethod
    def get_store(self, store_url: str) -> Dict[Text, Any]:
        pass


class TabelogRecommender:
    def __init__(self) -> None:
        self.extractor = inject.instance(Extractor)

    def execute(self, ignore_urls) -> Tuple[Dict[Text, Any]]:
        # 分析対象となる店舗URLをすべて取得
        store_urls = self.extractor.get_store_urls()

        # 店舗URLをひとつずつ回しながら詳細情報を取得
        store_list: List[Dict[Text, Any]] = []
        for url in store_urls:
            if url in ignore_urls:
                continue
            store = self.extractor.get_store(url)
            store_list.append(store)

        return tuple(store_list)



import inject
from tabelog_scraper.adapter.tabelog_recommender import Extractor
from tabelog_scraper.adapter.extractor.scrapeing_extractor import ScrapingExtractor

__target_url = ''
__limit_page_count = 0


def init_injection(target_url: str, limit_page_count: int):
    global __target_url, __limit_page_count
    __target_url = target_url
    __limit_page_count = limit_page_count
    inject.configure(inject_config)


def inject_config(binder):
    scraping_extractor = ScrapingExtractor(__target_url, __limit_page_count)
    # インスタンスを直で指定する場合は bind を使う
    binder.bind(Extractor, scraping_extractor)
    # 継承先のクラスを指定する場合は bind_to_constructor を使う



from setuptools import setup, find_packages

setup(
    name="tabelog_scraper",
    version='1.2',
    description='食べログのスクレイピング',
    author='Kobori Akira',
    author_email='private.beats@gmail.com',
    url='https://github.com/koboriakira/tabelog_scraper',
    packages=find_packages(),
    install_require=open('requirements.txt').read().splitlines(),
    license='MIT',
)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""メインモジュール
"""
from typing import Dict, Tuple, Text, Any, List
from tabelog_scraper.config.inject_config import init_injection
from tabelog_scraper.adapter.tabelog_recommender import TabelogRecommender


def scrape(target_url: str, limit_page_count: int = 10,
           ignore_urls: List[str] = []) -> Tuple[Dict[Text, Any]]:
    """スクレイピング処理
    スクレイピング処理を実行。時間がかかることに注意

    Args:
        target_url (:obj:`str`): スクレイピングしたい食べログページを指定。1ページ目を利用すること。
        limit_page_count (:obj:`int`): スクレイピングする上限ページを指定。
        ignore_urls (:obj:`List[str]`): スクレイピング対象にしない店舗URLを指定。
    Returns:
        Tuple[Dict[Text, Any]]: 取得できたDict型の店舗情報リストをタプル型で返却
    """
    # 依存注入を整理
    init_injection(target_url, limit_page_count)

    # 実行
    return TabelogRecommender().execute(ignore_urls)


if __name__ == '__main__':
    """
    テスト実行用
    """
    result = scrape(
        "https://tabelog.com/tokyo/A1315/A131501/R1644/rstLst/1/?svd=20200313&svt=1900&svps=2",
        1)
    print(result)
